@@ -34,75 +34,9 @@ is sufficient. Inside a language definition, it is easier to just use::
 
 to get easy access to all the actions and the helper functions.
 
-Besides the classes and functions below, the following standard actions are
-defined here. See the :py:mod:`action <parce.action>` module for more
-explanation abount standard actions.
-
-Generic actions:
-
-    .. py:data:: Whitespace
-    .. py:data:: Text
-
-Base actions:
-
-    .. py:data:: Comment
-    .. py:data:: Delimiter
-    .. py:data:: Error
-    .. py:data:: Escape
-    .. py:data:: Keyword
-    .. py:data:: Literal
-    .. py:data:: Name
-    .. py:data:: Pseudo
-    .. py:data:: Template
-
-Actions that derive from :py:data:`Name`:
-
-    .. py:data:: Name.Attribute
-    .. py:data:: Name.Builtin
-    .. py:data:: Name.Class
-    .. py:data:: Name.Command
-    .. py:data:: Name.Constant
-    .. py:data:: Name.Decorator
-    .. py:data:: Name.Exception
-    .. py:data:: Name.Function
-    .. py:data:: Name.Identifier
-    .. py:data:: Name.Macro
-    .. py:data:: Name.Method
-    .. py:data:: Name.Namespace
-    .. py:data:: Name.Object
-    .. py:data:: Name.Property
-    .. py:data:: Name.Symbol
-    .. py:data:: Name.Tag
-    .. py:data:: Name.Variable
-
-Actions that derive from :py:data:`Literal`:
-
-    .. py:data:: Verbatim( = Literal.Verbatim)
-    .. py:data:: String( = Literal.String)
-    .. py:data:: Number( = Literal.Number)
-    .. py:data:: Boolean( = Literal.Boolean)
-    .. py:data:: Char( = Literal.Char)
-
-Other derived actions:
-
-    .. py:data:: Comment.Alert
-    .. py:data:: Literal.Color
-    .. py:data:: Literal.Email
-    .. py:data:: Literal.Url
-    .. py:data:: Operator(= Delimiter.Operator)
-    .. py:data:: String.Double
-    .. py:data:: String.Single
-    .. py:data:: String.Escape
-    .. py:data:: Template.Preprocessed
-    .. py:data:: Text.Deleted
-    .. py:data:: Text.Inserted
-
-If you reference a non-existing sub-action, it is created.
-
-When highlighting, a standard action maps to a list of CSS classes with all the
-names lowercased. So String.Double maps to the ("literal", "string", "double")
-CSS classes. See the :py:mod:`theme <parce.theme>` module for more
-information.
+Besides the classes and functions below, a large amount of *standard actions* is
+also available in the ``parce`` module namespace. See for the full list
+:doc:`stdactions`.
 
 """
 
@@ -112,22 +46,31 @@ from . import lexicon as lexicon_
 from .document import Cursor
 from .language import Language
 from .pkginfo import version, version_string
+from .stdactions import *
 
 
 class Document(treedocument.TreeDocumentMixin, document.Document):
-    """A Document that automatically keeps its contents tokenized."""
-    def __init__(self, root_lexicon=None, text=""):
+    """A Document that automatically keeps its contents tokenized.
+
+    You can specify your own TreeBuilder. By default, a BackgroundTreeBuilder
+    is used.
+
+    """
+    def __init__(self, root_lexicon=None, text="", builder=None):
         document.Document.__init__(self, text)
-        builder = treebuilder.BackgroundTreeBuilder(root_lexicon)
+        if builder is None:
+            builder = treebuilder.BackgroundTreeBuilder(root_lexicon)
+        else:
+            builder.root.clear()
+            builder.root.lexicon = root_lexicon
         treedocument.TreeDocumentMixin.__init__(self, builder)
         if text:
-            with builder.change() as c:
-                c.change_contents(text)
+            builder.rebuild(text)
 
 
 def root(root_lexicon, text):
     """Return the root context of the tree structure of all tokens from text."""
-    return treebuilder.TreeBuilder(root_lexicon).tree(text)
+    return treebuilder.build_tree(root_lexicon, text)
 
 
 def tokens(root_lexicon, text):
@@ -141,7 +84,8 @@ def events(root_lexicon, text):
 
 
 def words(words, prefix="", suffix=""):
-    r"""Return a :class:`~parce.pattern.Pattern` matching any of the words.
+    r"""Return a :class:`~parce.pattern.Words` pattern matching any of the
+    words.
 
     The returned Pattern builds an optimized regular expression matching any of
     the words contained in the `words` list.
@@ -155,8 +99,8 @@ def words(words, prefix="", suffix=""):
 
 
 def char(chars, positive=True):
-    """Return a :class:`~parce.pattern.Pattern` matching one of the characters
-    in the specified string.
+    """Return a :class:`~parce.pattern.Char` pattern matching one of the
+    characters in the specified string.
 
     If `positive` is False, the set of characters is complemented, i.e. the
     Pattern matches any single character that is not in the specified string.
@@ -166,8 +110,8 @@ def char(chars, positive=True):
 
 
 def arg(escape=True, prefix="", suffix="", default=None):
-    r"""Return a :class:`~parce.pattern.Pattern` that contains the argument the
-    current Lexicon was called with.
+    r"""Return an :class:`~parce.pattern.Arg` pattern that contains the
+    argument the current Lexicon was called with.
 
     If there is no argument in the current lexicon, this Pattern yields the
     default value, which is by default None, resulting in the rule being
@@ -179,18 +123,11 @@ def arg(escape=True, prefix="", suffix="", default=None):
     ``suffix`` are not used.
 
     """
-    import re
-    def predicate(arg):
-        if isinstance(arg, str):
-            if escape:
-                arg = re.escape(arg)
-            return prefix + arg + suffix
-        return default
-    return pattern.PredicatePattern(predicate)
+    return pattern.Arg(escape, prefix, suffix, default)
 
 
 def ifarg(pattern, else_pattern=None):
-    r"""Return a :class:`~parce.pattern.Pattern` that only yields the specified
+    r"""Return an :class:`~parce.pattern.IfArg` pattern yielding the specified
     regular expression pattern (or nested Pattern instance) if the lexicon was
     called with an argument.
 
@@ -198,56 +135,54 @@ def ifarg(pattern, else_pattern=None):
     yielded, which is None by default, resulting in the rule being skipped.
 
     """
-    def predicate(arg):
-        return pattern if arg is not None else else_pattern
-    return pattern.PredicatePattern(predicate)
+    return pattern.IfArg(pattern, else_pattern)
 
 
 def byarg(predicate, *itemlists):
-    """Return an :class:`~parce.rule.ArgRuleItem` that chooses its output based
-    on the lexicon argument.
+    """Return an :class:`~parce.rule.PredicateArgItem` that chooses its output
+    based on the lexicon argument.
 
     The predicate is called with the lexicon argument (which is None for a
     normal Lexicon, but can have another value for a derivative Lexicon.
 
     """
-    return rule.ArgRuleItem(predicate, *itemlists)
+    return rule.PredicateArgItem(predicate, *itemlists)
 
 
 def bymatch(predicate, *itemlists):
-    """Return a :class:`~parce.rule.MatchRuleItem` that chooses its output
+    """Return a :class:`~parce.rule.MatchItem` that chooses its output
     based on the match object.
 
-    The returned MatchRuleItem calls the predicate function with the match
-    object as argument. The function should return the index of the itemlist
-    to choose. If you provide two possible actions, the predicate function
-    may also return True or False, in which case True chooses the second
-    itemlist and False the first.
+    The returned MatchItem calls the predicate function with the match object
+    as argument. The function should return the index of the itemlist to
+    choose. If you provide two possible actions, the predicate function may
+    also return True or False, in which case True chooses the second itemlist
+    and False the first.
 
     This helper can be used both for action and target objects, or both
     at the same time.
 
     """
-    return rule.MatchRuleItem(predicate, *itemlists)
+    return rule.MatchItem(predicate, *itemlists)
 
 
 def bytext(predicate, *itemlists):
-    """Return a :class:`~parce.rule.TextRuleItem` that chooses the itemlist
+    """Return a :class:`~parce.rule.TextItem` that chooses the itemlist
     based on the text.
 
-    The returned TextRuleItem calls the predicate function with the matched
-    text as argument.  The function should return the index of the itemlist
-    to choose, in the same way as with :func:`~parce.bymatch`.
+    The returned TextItem calls the predicate function with the matched text as
+    argument.  The function should return the index of the itemlist to choose,
+    in the same way as with :func:`~parce.bymatch`.
 
     This helper can be used both for action and target objects, or both
     at the same time.
 
     """
-    return rule.TextRuleItem(predicate, *itemlists)
+    return rule.TextItem(predicate, *itemlists)
 
 
 def ifgroup(n, itemlist, else_itemlist=()):
-    r"""Return a :class:`~parce.rule.MatchRuleItem` that yields ``itemlist`` if
+    r"""Return a :class:`~parce.rule.MatchItem` that yields ``itemlist`` if
     group n in the match is not empty.
 
     If group ``n`` in the match object is empty, ``else_itemlist`` is yielded.
@@ -269,7 +204,7 @@ def ifgroup(n, itemlist, else_itemlist=()):
 
 
 def ifmember(sequence, itemlist, else_itemlist=()):
-    r"""Return a :class:`~parce.rule.TextRuleItem` that yields ``itemlist`` if
+    r"""Return a :class:`~parce.rule.TextItem` that yields ``itemlist`` if
     the text is in sequence.
 
     If text is not in sequence, ``else_itemlist`` is yielded.
@@ -283,22 +218,83 @@ def ifmember(sequence, itemlist, else_itemlist=()):
     with the Keyword action, and only for those, switches to the
     ``cls.keyword`` lexicon. Other words get the Name.Variable action.
 
+    Internally this helper function creates a frozenset of the sequence, to
+    speed up membership testing.
+
     """
-    def predicate(text):
-        return text in sequence
-    return rule.TextRuleItem(predicate, else_itemlist, itemlist)
+    sequence_set = frozenset(sequence)
+    predicate = sequence_set.__contains__
+    return bytext(predicate, else_itemlist, itemlist)
 
 
 def ifgroupmember(n, sequence, itemlist, else_itemlist=()):
-    """Return a :class:`~parce.rule.MatchRuleItem` that yields ``itemlist`` if
+    """Return a :class:`~parce.rule.MatchItem` that yields ``itemlist`` if
     group ``n`` is in sequence.
 
     If group ``n`` is not in sequence, ``else_itemlist`` is yielded.
 
+    Internally this helper function creates a frozenset of the sequence, to
+    speed up membership testing.
+
     """
+    sequence_set = frozenset(sequence)
     def predicate(m):
-        return m.group(m.lastindex + n) in sequence
-    return rule.MatchRuleItem(predicate, else_itemlist, itemlist)
+        return m.group(m.lastindex + n) in sequence_set
+    return bymatch(predicate, else_itemlist, itemlist)
+
+
+def _get_sequences_map(pairs, default):
+    """Map sequences to itemlists, with a default value.
+
+    Returns a predicate that tries to find text in sequences (the first one
+    that has a match is chosen) and the itemlists.
+
+    Optimizes by making a frozenset of each sequence.
+
+    """
+    sequences = []
+    itemlists = []
+    all_items = set()
+    for s, i in pairs:
+        s = frozenset(set(s) - all_items)
+        all_items |= s
+        sequences.append(s)
+        itemlists.append(i)
+    last = len(itemlists)
+    itemlists.append(default)
+    def predicate(text):
+        for i, sequence in enumerate(sequences):
+            if text in sequence:
+                return i
+        return last
+    return predicate, itemlists
+
+
+def mapmember(pairs, default=()):
+    r"""Return a :class:`~parce.rule.TextItem` that yields ``itemlist`` from
+    a pair's itemlist if the text is in the pair's sequence.
+
+    ``pairs`` can be a list of (sequence, itemlist) values, or a (ordered)
+    dictionary.
+
+    """
+    predicate, itemlists = _get_sequences_map(pairs, default)
+    return bytext(predicate, *itemlists)
+
+
+def mapgroupmember(n, pairs, default=()):
+    r"""Return a :class:`~parce.rule.MatchItem` that yields ``itemlist`` from
+    a pair's itemlist if the matched group is in the pair's sequence.
+
+    ``pairs`` can be a list of (sequence, itemlist) values, or a (ordered)
+    dictionary.
+
+    """
+    text_predicate, itemlists = _get_sequences_map(pairs, default)
+    def predicate(m):
+        return text_predicate(m.group(m.lastindex + n))
+    return bymatch(predicate, *itemlists)
+
 
 
 def _get_items_map(dictionary, default):
@@ -317,7 +313,7 @@ def _get_items_map(dictionary, default):
 
 
 def maptext(dictionary, default=()):
-    r"""Return a :class:`~parce.rule.TextRuleItem` that yields the itemlist
+    r"""Return a :class:`~parce.rule.TextItem` that yields the itemlist
     from the dictionary, using the text as key.
 
     If the dict does not contain the key, the default value is yielded.
@@ -335,11 +331,11 @@ def maptext(dictionary, default=()):
 
     """
     predicate, itemlists = _get_items_map(dictionary, default)
-    return rule.TextRuleItem(predicate, *itemlists)
+    return bytext(predicate, *itemlists)
 
 
 def mapgroup(n, dictionary, default=()):
-    """Return a :class:`~parce.rule.MatchRuleItem` that yields the itemlist
+    """Return a :class:`~parce.rule.MatchItem` that yields the itemlist
     from the dictionary, using the specified match group as key.
 
     If the dict does not contain the key, the default value is yielded.
@@ -348,7 +344,7 @@ def mapgroup(n, dictionary, default=()):
     get, itemlists = _get_items_map(dictionary, default)
     def predicate(m):
         return get(m.group(m.lastindex + n))
-    return rule.MatchRuleItem(predicate, *itemlists)
+    return bymatch(predicate, *itemlists)
 
 
 def bygroup(*actions):
@@ -373,8 +369,8 @@ def bygroup(*actions):
 
 
 def withgroup(n, lexicon, mapping=None):
-    r"""Return a :class:`~parce.rule.LexiconWithGroup` rule item that calls the
-    ``lexicon`` with the matched text from group ``n``.
+    r"""Return a :class:`~parce.rule.LexiconMatchItem` rule item that calls
+    the ``lexicon`` with the matched text from group ``n``.
 
     Calling a Lexicon creates a derived Lexicon, i.e. one that has the same set
     of rules and the same name, but the patterns and/or rules may differ by
@@ -386,27 +382,37 @@ def withgroup(n, lexicon, mapping=None):
     the argument to call the lexicon with.
 
     """
-    return rule.LexiconWithGroup(n, lexicon, mapping)
+    if mapping:
+        def predicate(match):
+            return mapping.get(match.group(match.lastindex + n))
+    else:
+        def predicate(match):
+            return match.group(match.lastindex + n)
+    return rule.LexiconMatchItem(predicate, lexicon)
 
 
 def withtext(lexicon, mapping=None):
-    r"""Return a :class:`~parce.rule.LexiconWithText` rule item that calls the
-    ``lexicon`` with the matched text.
+    r"""Return a :class:`~parce.rule.LexiconTextItem` rule item that calls
+    the ``lexicon`` with the matched text.
 
     If a ``mapping`` dictionary is specified, the matched text is used as key,
     and the result of the mapping (None if not present) gives the argument to
     call the lexicon with.
 
     """
-    return rule.LexiconWithText(lexicon, mapping)
+    predicate = mapping.get if mapping else lambda t: t
+    return rule.LexiconTextItem(predicate, lexicon)
 
 
 def witharg(lexicon):
-    r"""Return a :class:`~parce.rule.LexiconWithArg` rule item that calls the
-    ``lexicon`` with the same argument as the current lexicon.
+    r"""Return a :class:`~parce.rule.LexiconArgItem` rule item that calls the
+    ``lexicon`` with the current lexicon's argument.
+
+    Use this to get a derived lexicon with the same argument as the current
+    lexicon.
 
     """
-    return rule.LexiconWithArg(lexicon)
+    return rule.LexiconArgItem(lexicon)
 
 
 def lexicon(rules_func=None, **kwargs):
@@ -459,61 +465,3 @@ default_target = object()   #: denotes a default target when no text matches
 #: used to suppress generating a token
 skip = action.SkipAction()
 
-# predefined standard actions
-# keep these in sync with the list above in the doc string.
-Whitespace = action.StandardAction("Whitespace")
-Text = action.StandardAction("Text")
-
-Comment = action.StandardAction("Comment")
-Delimiter = action.StandardAction("Delimiter")
-Error = action.StandardAction("Error")
-Escape = action.StandardAction("Escape")
-Keyword = action.StandardAction("Keyword")
-Literal = action.StandardAction("Literal")
-Name = action.StandardAction("Name")
-Pseudo = action.StandardAction("Pseudo")
-Template = action.StandardAction("Template")
-
-# Actions that derive from Name
-
-Name.Attribute
-Name.Builtin
-Name.Class
-Name.Command
-Name.Constant
-Name.Decorator
-Name.Exception
-Name.Function
-Name.Identifier
-Name.Macro
-Name.Method
-Name.Namespace
-Name.Object
-Name.Property
-Name.Symbol
-Name.Tag
-Name.Variable
-
-# Actions that derive from Literal:
-
-Verbatim = Literal.Verbatim
-String = Literal.String
-Number = Literal.Number
-Boolean = Literal.Boolean
-Char = Literal.Char
-
-# Other derived actions:
-
-Comment.Alert
-Literal.Color
-Literal.Email
-Literal.Url
-Operator = Delimiter.Operator
-String.Double
-String.Single
-String.Escape
-Template.Preprocessed
-Text.Deleted
-Text.Inserted
-
-
