@@ -112,6 +112,7 @@ class LilyPond(Language):
         yield r'\}', Delimiter.CloseBrace, -1
         yield from cls.common()
         yield from cls.blocks()
+        yield from cls.music()
 
     @lexicon
     def score(cls):
@@ -165,7 +166,6 @@ class LilyPond(Language):
     @classmethod
     def commands(cls):
         """Yield commands that can occur in all input modes."""
-        pitch = cls.ifpitch()
         yield RE_LILYPOND_DYNAMIC, Dynamic
         yield r"(\\repeat)\b(?:\s+([a-z]+)\s*(\d+)?)?", bygroup(Keyword, Name.Symbol, Number)
         yield r"\\(?:un)?set(?![^\W\d])", Keyword, cls.set_unset
@@ -173,34 +173,21 @@ class LilyPond(Language):
         yield r"\\(?:new|change|context)(?![^\W\d])", Keyword, cls.context
         yield r"(\\with)\s*(\{)", bygroup(Keyword, Delimiter.OpenBrace), cls.layout_context
         yield r"(\\relative)(?![^\W\d])(?:\s+" + RE_LILYPOND_PITCH_OCT + ")?", \
-            bygroup(Keyword, pitch, Octave)
+            bygroup(Keyword, cls.ifpitch(), Octave)
         yield (r"(\\transpose)(?![^\W\d])(?:\s+" + RE_LILYPOND_PITCH_OCT + ")?"
                r"(?:\s*" + RE_LILYPOND_PITCH_OCT + ")?"), \
-               bygroup(Keyword, pitch, Octave, pitch, Octave)
+               bygroup(Keyword, cls.ifpitch(), Octave, cls.ifpitch(), Octave)
         yield r"\\tempo(?![^\W\d])", Keyword, cls.tempo
         yield r"(\\chord(?:s|mode))\b\s*(\{)?", bygroup(Keyword, Delimiter.OpenBrace), \
             ifgroup(2, cls.chordmode)
         yield from cls.notemode_rule()
         yield from cls.lyricmode_rules()
         yield from cls.drummode_rule()
-        yield RE_LILYPOND_COMMAND, cls.check_builtin()
-
-    @classmethod
-    def check_builtin(cls):
-        """Return a suitable action for a \\command.
-
-        When the command is a member of lilypond_keywords Keyword is returned,
-        When the command is in the music_commands, Name.Builtin is returned,
-        otherwise Name.Command.
-
-        """
-        def predicate(text):
-            text = text[1:] # skip the "\"
-            return 0 if text in lilypond_words.keywords else \
-                   1 if text in lilypond_words.music_commands_set else \
-                   2 if text in lilypond_words.articulations_set else \
-                   3
-        return bytext(predicate, Keyword, Name.Builtin, Articulation, Name.Command)
+        yield RE_LILYPOND_COMMAND, mapgroupmember(1, (
+            (lilypond_words.keywords, Keyword),
+            (lilypond_words.music_commands, Name.Builtin),
+            (lilypond_words.all_articulations, Articulation),
+            ), Name.Command)
 
     # ------------------ music ----------------------
     @classmethod
@@ -302,7 +289,7 @@ class LilyPond(Language):
             itemlist = Name.Pitch
         if else_itemlist is None:
             else_itemlist = Name.Symbol
-        return ifmember(lilypond_words.all_pitchnames, itemlist, else_itemlist)
+        return ifmember(lilypond_words.all_pitch_names, itemlist, else_itemlist)
 
     @lexicon
     def pitch(cls):
@@ -318,13 +305,14 @@ class LilyPond(Language):
         """ zero or more dots after a duration. """
         yield SKIP_WHITESPACE
         yield r'\.', Duration.Dot
+        #yield from cls.comments()
         yield default_target, cls.duration_scaling
 
     @lexicon
     def duration_scaling(cls):
         """ * n / m after a duration. """
         yield SKIP_WHITESPACE
-        yield from cls.comments()
+        #yield from cls.comments()
         yield r"(\*)\s*(\d+(?:/\d+)?)", bygroup(Duration, Duration.Scaling)
         yield default_target, -2
 
@@ -390,7 +378,7 @@ class LilyPond(Language):
         yield r"<<", Delimiter.OpenBrace, cls.drummode_simultaneous
         yield r"\}|>>", Delimiter.CloseBrace, -1
         yield RE_LILYPOND_REST, Rest
-        yield RE_LILYPOND_PITCHWORD, ifmember(lilypond_words.drum_pitchnames_set, Pitch.Drum, Name.Symbol)
+        yield RE_LILYPOND_PITCHWORD, ifmember(lilypond_words.drum_pitch_names_set, Pitch.Drum, Name.Symbol)
         yield from cls.music()
 
     @lexicon
@@ -453,7 +441,7 @@ class LilyPond(Language):
     @lexicon
     def markup(cls):
         """Markup without environment. Try to guess the n of arguments."""
-        yield r'\{', Delimiter.OpenBrace, -1, cls.markup_environ
+        yield r'\{', Delimiter.OpenBrace, -1, cls.markuplist
         yield RE_LILYPOND_COMMAND, cls.get_markup_action(), cls.get_markup_target()
         yield r'"', String, -1, cls.string
         yield r'[#$]', Delimiter.SchemeStart, -1, cls.get_scheme_target()
@@ -472,12 +460,12 @@ class LilyPond(Language):
     def get_markup_argument_count(cls, command):
         """Return the number of arguments the markup command (without \\) expects."""
         for i in range(5):
-            if command in lilypond_words.markupcommands_nargs[i]:
+            if command in lilypond_words.markup_commands_nargs[i]:
                 return i
         return 1    # assume a user command has no arguments
 
     @lexicon
-    def markup_environ(cls):
+    def markuplist(cls):
         """Markup until } ."""
         yield r'\}', Delimiter.CloseBrace, -1
         yield r'\{', Delimiter.OpenBrace, 1
@@ -488,7 +476,7 @@ class LilyPond(Language):
     @classmethod
     def get_markup_action(cls):
         r"""Get the action for a command in \markup { }."""
-        return ifmember(lilypond_words.markupcommands, Name.Function.Markup, Name.Function)
+        return ifmember(lilypond_words.markup_commands, Name.Function.Markup, Name.Function)
 
     # -------------- Scheme ---------------------
     @classmethod
