@@ -17,8 +17,21 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
+"""
+This module provides the Language class, which serves as the base class
+for all language definitions.
+
+Additionally, there are some utility functions to manage and query built-in
+language definitions.
+
+"""
+
+import importlib
+import glob
+import os
 
 import parce
+import parce.lang
 from parce.action import StandardAction
 from parce.rule import DynamicItem
 from parce.lexicon import LexiconDescriptor, Lexicon
@@ -81,3 +94,43 @@ def languages(lang):
         for i in rule_items(lang)
             if isinstance(i, Lexicon) and i.language is not lang)
 
+
+def get_all_modules():
+    """Return the sorted list of module names in ``parce.lang``.
+
+    Modules that start with an underscore are skipped.
+
+    """
+    names = []
+    for filename in glob.glob(os.path.join(parce.lang.__path__[0], "*.py")):
+        name = os.path.splitext(os.path.basename(filename))[0]
+        if not name.startswith('_'):
+            names.append(name)
+    names.sort()
+    return names
+
+
+def get_languages(name):
+    """Yield the Language subclasses defined in the module ``name``.
+
+    The module name must be one of the modules returned by
+    :meth:``get_all_modules``.
+
+    """
+    modname = 'parce.lang.' + name
+    mod = importlib.import_module(modname)
+    for name, obj in mod.__dict__.items():
+        if (isinstance(obj, type) and issubclass(obj, parce.Language)
+               and obj is not parce.Language and obj.__module__ == modname):
+            yield obj
+
+
+def get_all_languages():
+    """Import all modules in ``parce.lang`` and yield all defined classes
+    that inherit from :class:`Language`.
+
+    """
+    for name in get_all_modules():
+        yield from get_languages(name)
+
+
