@@ -38,10 +38,10 @@ import itertools
 import re
 import threading
 
-import parce.pattern
 import parce.regex
+from .pattern import Pattern
 from .target import TargetFactory
-from .rule import DynamicItem, DynamicRuleItem, ArgRuleItem
+from .rule import Item, ArgItem, DynamicItem, variations
 
 
 class LexiconDescriptor:
@@ -88,12 +88,11 @@ class Lexicon:
     This function is created as soon as it is called for the first time.
 
     """
-    __slots__ = ('lexicon', 'language', 'arg', 'parse', '_lock', '_derived')
-
     def __init__(self, lexicon, language, arg=None):
         self.lexicon = lexicon
         self.language = language
         self.arg = arg
+        self.__doc__ = lexicon.rules_func.__doc__
         self._derived = {}
         # lock is used when creating a derivate and/or the parse() instance function
         self._lock = threading.Lock()
@@ -132,14 +131,14 @@ class Lexicon:
         return self.lexicon is other.lexicon and self.language is other.language
 
     def __iter__(self):
-        """Yield the rules, replacing the ArgRuleItem instances."""
+        """Yield the rules, replacing the ArgItem instances."""
         def replace_arg_items(items):
             """Replace ArgRuleItem instances."""
             for i in items:
-                if isinstance(i, ArgRuleItem):
+                if isinstance(i, ArgItem):
                     yield from replace_arg_items(i.replace(self.arg))
                 else:
-                    if isinstance(i, DynamicItem):
+                    if isinstance(i, Item):
                         i.itemlists = [list(replace_arg_items(l)) for l in i.itemlists]
                     yield i
         for rule in self.lexicon.rules_func(self.language) or ():
@@ -156,19 +155,19 @@ class Lexicon:
         return '.'.join((self.language.__name__, self.lexicon.rules_func.__name__))
 
     def __getattr__(self, name):
-        """Implemented to create the parse() function when called for the first time.
+        """Create certain instance attributes when requested the first time.
 
-        The function is created by _get_parser_func() and is set as instance
-        variable, so __getattr__ is not called again.
+        Calls :meth:`get_instance_attributes` to get instance attributes needed
+        to use the Lexicon. Those attributes then are set in the Lexicon
+        instance, so the do not need to be computed again.
 
         """
-        if name == "parse":
+        if name in ("parse",):
             with self._lock:
                 try:
                     return object.__getattribute__(self, name)
                 except AttributeError:
-                    self.parse = self._get_parser_func()
-                    return self.parse
+                    self.parse = self.get_instance_attributes()
         return object.__getattribute__(self, name)
 
     @property
@@ -176,21 +175,31 @@ class Lexicon:
         """The re_flags set on instantiation."""
         return self.lexicon.re_flags
 
-    def _get_parser_func(self):
-        """Compile the pattern rules and return a parse(text, pos) func."""
+    def get_instance_attributes(self):
+        """Compile the pattern rules and return instance attributes.
+
+        These are:
+
+        ``parse``
+            A ``parse(text, pos)`` function that parses text.
+
+        """
         patterns = []
         rules = []
-        default_action = None
+        no_default_action = object()
+        default_action = no_default_action
         default_target = None
 
+        make_target = TargetFactory.make
+
         # make lists of pattern, action and possible targets
         for pattern, *rule in self:
-            while isinstance(pattern, parce.pattern.Pattern):
-                pattern = pattern.build()
+            while isinstance(pattern, Pattern):
+                pattern = pattern.build(self.arg)
             if pattern is parce.default_action:
                 default_action = rule[0]
             elif pattern is parce.default_target:
-                default_target = TargetFactory.make(self, rule)
+                default_target = make_target(self, rule)
             elif rule and pattern is not None and pattern not in patterns:
                 # skip rule when the pattern is None or already seen
                 patterns.append(pattern)
@@ -198,7 +207,7 @@ class Lexicon:
 
         # handle the empty lexicon case
         if not patterns:
-            if default_action:
+            if default_action is not no_default_action:
                 def parse(text, pos):
                     yield pos, text[pos:], None, default_action, None
             elif default_target:
@@ -213,14 +222,14 @@ class Lexicon:
 
         # if there is only one pattern, and no dynamic action or target,
         # see if the pattern is simple enough to just use str.find
-        if len(patterns) == 1 and not any(isinstance(item, DynamicItem)
+        if len(patterns) == 1 and not any(isinstance(item, Item)
                                           for item in rules[0]):
             needle = parce.regex.to_string(patterns[0])
             if needle:
                 l= len(needle)
                 action, *rule = rules[0]
-                target = TargetFactory.make(self, rule)
-                if default_action:
+                target = make_target(self, rule)
+                if default_action is not no_default_action:
                     def parse(text, pos):
                         """Parse text, using a default action for unknown text."""
                         while True:
@@ -256,16 +265,16 @@ class Lexicon:
         rx = re.compile("|".join("(?P<g_{0}>{1})".format(i, pattern)
             for i, pattern in enumerate(patterns)), self.re_flags)
         # make a fast mapping list from matchObj.lastindex to the rules.
-        # rules that contain DynamicRuleItem instances are put in the dynamic index
+        # rules that contain Item instances are put in the dynamic index
         indices = sorted(v for k, v in rx.groupindex.items() if k.startswith('g_'))
         static = [None] * (indices[-1] + 1)
         dynamic = [None] * (indices[-1] + 1)
         for i, rule in zip(indices, rules):
-            if any(isinstance(item, DynamicRuleItem) for item in rule):
+            if any(isinstance(item, Item) for item in rule):
                 dynamic[i] = rule
             else:
                 action, *target = rule
-                static[i] = (action, TargetFactory.make(self, target))
+                static[i] = (action, make_target(self, target))
 
         # for rule containing no dynamic stuff, static has the rule, otherwise
         # falls back to dynamic, which is then immediately executed
@@ -277,17 +286,18 @@ class Lexicon:
             """Recursively replace dynamic rule items in the rule pointed to by match object."""
             def inner_replace(items):
                 for i in items:
-                    if isinstance(i, DynamicRuleItem):
+                    if isinstance(i, DynamicItem):
                         yield from inner_replace(i.replace(m.group(), m))
                     else:
                         yield i
             action, *target = inner_replace(dynamic[m.lastindex])
-            return action, TargetFactory.make(self, target)
+            return action, make_target(self, target)
 
-        if default_action:
+        if default_action is not no_default_action:
+            finditer = rx.finditer
             def parse(text, pos):
                 """Parse text, using a default action for unknown text."""
-                for m in rx.finditer(text, pos):
+                for m in finditer(text, pos):
                     if m.start() > pos:
                         yield pos, text[pos:m.start()], None, default_action, None
                     yield token(m)
@@ -295,10 +305,11 @@ class Lexicon:
                 if pos < len(text):
                     yield pos, text[pos:], None, default_action, None
         elif default_target:
+            match = rx.match
             def parse(text, pos):
                 """Parse text, stopping with the default target at unknown text."""
                 while True:
-                    m = rx.match(text, pos)
+                    m = match(text, pos)
                     if m:
                         yield token(m)
                         pos = m.end()
@@ -307,10 +318,10 @@ class Lexicon:
                             yield pos, "", None, None, default_target
                         break
         else:
+            finditer = rx.finditer
             def parse(text, pos):
                 """Parse text, skipping unknown text."""
-                for m in rx.finditer(text, pos):
-                    yield token(m)
+                return map(token, finditer(text, pos))
         return parse
 
 
