@@ -35,7 +35,7 @@ Version = collections.namedtuple("Version", "major minor patch")
 name = "parce"
 
 #: the current version
-version = Version(0, 7, 0)
+version = Version(0, 8, 0)
 version_suffix = ""
 version_string = "{}.{}.{}".format(*version) + version_suffix
 
@@ -53,7 +53,7 @@ maintainer = "Wilbert Berendsen"
 maintainer_email = "info@wilbertberendsen.nl"
 
 #: homepage
-url = "https://github.com/wbsoft/parce"
+url = "https://parce.info/"
 
 #: license
 license = "GPL"
