@@ -19,8 +19,7 @@
 
 
 r"""
-Querying the tree using the `query` property.
----------------------------------------------
+Query the tree using the `query` property.
 
 Using this module you can query the token tree to find tokens and contexts,
 based on lexicons and/or actions and text contents. You can chain calls
@@ -241,10 +240,10 @@ class Query:
         """Compute the length of the iterable."""
         return sum(1 for _ in self)
 
-    def dump(self):
-        """Dump the current selection to the console."""
+    def dump(self, file=None):
+        """Dump the current selection to the console (or to file)."""
         for n in self:
-            n.dump()
+            n.dump(file)
 
     def list(self):
         """Return the current selection as a list. Mainly for debugging."""
@@ -284,7 +283,7 @@ class Query:
         """Delete all selected nodes from their parents.
 
         Internally calls ``uniq`` and ``remove_descendants``, so that no
-        unnessary deletes are done. If a context would become empty, that
+        unnecessary deletes are done. If a context would become empty, that
         context itself is deleted instead of all its children (except for the
         root of course). Returns the number of nodes that were deleted.
 
