@@ -48,14 +48,23 @@ See also the documentation for Token and Context.
 """
 
 
-import sys
 import itertools
 
 from parce import util
 from parce import query
 from parce.lexicon import Lexicon
-from parce.lexer import Event
-from parce.target import TargetFactory
+
+
+DUMP_STYLES = {
+    "ascii":   (" | ", "   ", " |-", " `-"),
+    "round":   (" │ ", "   ", " ├╴", " ╰╴"),
+    "square":  (" │ ", "   ", " ├╴", " └╴"),
+    "double":  (" ║ ", "   ", " ╠═", " ╚═"),
+    "thick":   (" ┃ ", "   ", " ┣╸", " ┗╸"),
+    "flat":    ("│", " ", "├", "╰"),
+}
+
+DUMP_STYLE_DEFAULT = "round"
 
 
 class Node:
@@ -65,14 +74,25 @@ class Node:
     is_token = False
     is_context = False
 
-    def dump(self, depth=0):
-        """Display a graphical representation of the node and its contents."""
-        prefix = (" ╰╴" if self.is_last() else " ├╴") if depth else ""
+    def dump(self, file=None, style=None, depth=0):
+        """Display a graphical representation of the node and its contents.
+
+        The file object defaults to stdout, and the style to "round". You can
+        choose any style that's in the ``DUMP_STYLES`` dictionary.
+
+        """
+        i = 2
+        d = DUMP_STYLES[style or DUMP_STYLE_DEFAULT]
+        prefix = []
         node = self
-        for i in range(depth - 1):
+        for _ in range(depth):
+            prefix.append(d[i + int(node.is_last())])
             node = node.parent
-            prefix = ("   " if node.is_last() else " │ ") + prefix
-        print(prefix + repr(self))
+            i = 0
+        print("".join(reversed(prefix)) + repr(self), file=file)
+        if self.is_context:
+            for n in self:
+                n.dump(file, style, depth + 1)
 
     def parent_index(self):
         """Return our index in the parent.
@@ -169,6 +189,10 @@ class Node:
             if n in ancestors:
                 return n
 
+    def depth(self):
+        """Return the number of ancestors."""
+        return sum(1 for n in self.ancestors())
+
     def left_sibling(self):
         """Return the left sibling of this node, if any.
 
@@ -231,7 +255,7 @@ class Node:
 
         """
         for parent, index in self.ancestors_with_index(upto):
-            yield from tokens(parent[index+1:])
+            yield from util.tokens(parent[index+1:])
 
     def backward(self, upto=None):
         """Yield all Tokens in backward direction.
@@ -242,7 +266,7 @@ class Node:
         """
         for parent, index in self.ancestors_with_index(upto):
             if index:
-                yield from tokens_bw(parent[index-1::-1])
+                yield from util.tokens_bw(parent[index-1::-1])
 
     @property
     def query(self):
@@ -344,14 +368,19 @@ class Token(Node):
         self.text = text
         self.action = action
 
+    def copy(self, parent=None):
+        """Return a copy of the Token, but with the specified parent."""
+        return type(self)(parent, self.pos, self.text, self.action)
+
     def equals(self, other):
-        """Return True if the other Token has the same ``pos``, ``text`` and
-        ``action`` attributes and the same context ancestry (see
+        """Return True if the other Token has the same ``text`` and ``action``
+        attributes and the same context ancestry (see also
         :meth:`state_matches`).
 
+        Note that the ``pos`` attribute is not compared.
+
         """
-        return (self.pos == other.pos
-                and self.text == other.text
+        return (self.text == other.text
                 and self.action == other.action
                 and self.state_matches(other))
 
@@ -405,67 +434,7 @@ class Token(Node):
         context, start_trail, end_trail = self.common_ancestor_with_trail(other)
         if context:
             for context, slice_ in context.slices(start_trail, end_trail):
-                yield from tokens(context[slice_])
-
-    def events_until_including(self, other):
-        """Yield events for all tokens starting with us and upto and including the other."""
-        context, start_trail, end_trail = self.common_ancestor_with_trail(other)
-        if context:
-            yield from context.events(start_trail, end_trail)
-
-    def split(self):
-        """Split off a new tree, starting with this token.
-
-        The new tree has the same ancestor structure as the current. This token
-        and all tokens to the right are moved to the new tree and removed from
-        the current one. The new tree's root element is returned.
-
-        """
-        parent = self.parent
-        node = firstchild = self
-        for p, i in self.ancestors_with_index():
-            copy = Context(p.lexicon, None)
-            copy.append(firstchild)
-            if node is not p[-1]:
-                s = slice(i + 1, None)
-                for n in p[s]:
-                    n.parent = copy
-                copy.extend(p[s])
-                del p[s]
-            firstchild.parent = copy
-            firstchild = copy
-            node = p
-        del parent[-1]
-        # remove empty context in the old tree
-        while not parent and parent.parent:
-            parent = parent.parent
-            del parent[-1]
-        return copy
-
-    def join(self, context):
-        """Add ourselves and all tokens to the right to the context.
-
-        This method assumes that the context has the same parent depth
-        as our own, and only makes sense if those parents also have the same
-        lexicon, i.e. the our state matches the target context (and that
-        the pos attribute of the tokens is adjusted).
-
-        The nodes are not removed from their former parents, just the parent
-        attribute is changed.
-
-        """
-        context.append(self)
-        node = self
-        c = context
-        for p, i in self.ancestors_with_index():
-            if node is not p[-1]:
-                siblings = p[i+1:]
-                for n in siblings:
-                    n.parent = c
-                c.extend(siblings)
-            node = p
-            c = c.parent
-        self.parent = context
+                yield from util.tokens(context[slice_])
 
     def common_ancestor_with_trail(self, other):
         """Return a three-tuple(context, trail_self, trail_other).
@@ -575,11 +544,12 @@ class Context(list, Node):
 
     def __repr__(self):
         pos, end = self.pos, self.end
-        if pos is None:
-            pos = end = "?"
+        if pos == end:
+            pos = end = "?" # both are 0 in this case: empty Context
         name = self.lexicon and repr(self.lexicon)
-        return "<Context {} at {}-{} ({} children)>".format(
-            name, pos, end, len(self))
+        children = "child" if len(self) == 1 else "children"
+        return "<Context {} at {}-{} ({} {})>".format(
+            name, pos, end, len(self), children)
 
     def __hash__(self):
         return Node.__hash__(self)
@@ -594,46 +564,84 @@ class Context(list, Node):
             return not other.equals(self.lexicon)
         return other is not self
 
-    def dump(self, depth=0):
-        """Prints a nice graphical representation, for debugging purposes."""
-        super().dump(depth)
-        for n in self:
-            n.dump(depth + 1)
+    def copy(self, parent=None):
+        """Return a copy of the context, but with the specified parent."""
+        context = type(self)(self.lexicon, parent)
+        nodes = iter(self)
+        for node in nodes:
+            if node.is_token and node.group:
+                # copy grouped tokens correctly
+                group = node.copy(context),
+                count = len(node.group) - 1
+                group += tuple(t.copy(context) for t in itertools.islice(nodes, count))
+                for n in group:
+                    n.group = group
+                context.extend(group)
+            else:
+                context.append(node.copy(context))
+        return context
 
     @property
     def pos(self):
-        """Return the position or our first token. Returns None if empty."""
-        for t in self.tokens():
-            return t.pos
+        """Return the position or our first token. Returns 0 if empty."""
+        try:
+            node = self[0]
+            while node.is_context:
+                node = node[0]
+            return node.pos
+        except IndexError:
+            return 0
 
     @property
     def end(self):
-        """Return the end position or our last token. Returns None if empty."""
-        for t in self.tokens_bw():
-            return t.end
+        """Return the end position or our last token. Returns 0 if empty."""
+        try:
+            node = self[-1]
+            while node.is_context:
+                node = node[-1]
+            return node.end
+        except IndexError:
+            return 0
 
-    def tokens(self, start=0, end=None):
-        """Yield all tokens (that completely fill this text range if specified).
+    def height(self):
+        """Return the height of the tree (the longest distance to a descendant)."""
+        return max(n.height() + 1 if n.is_context else 1 for n in self) if self else 0
 
-        The first and last tokens may overlap with the start and end positions.
-
-        """
-        for context, slice_ in self.context_slices(start, end):
-            yield from tokens(context[slice_])
+    def tokens(self):
+        """Yield all Tokens, descending into nested Contexts."""
+        for n in self:
+            if n.is_token:
+                yield n
+            else:
+                yield from n.tokens()
 
     def tokens_bw(self):
         """Yield all Tokens, descending into nested Contexts, in backward direction."""
-        yield from tokens_bw(self[::-1])
+        for n in self[::-1]:
+            if n.is_token:
+                yield n
+            else:
+                yield from n.tokens_bw()
 
     def first_token(self):
         """Return our first Token."""
-        for t in self.tokens():
-            return t
+        try:
+            node = self[0]
+            while node.is_context:
+                node = node[0]
+            return node
+        except IndexError:
+            pass
 
     def last_token(self):
         """Return our last token."""
-        for t in self.tokens_bw():
-            return t
+        try:
+            node = self[-1]
+            while node.is_context:
+                node = node[-1]
+            return node
+        except IndexError:
+            pass
 
     def find(self, pos):
         """Return the index of our child at pos."""
@@ -642,7 +650,9 @@ class Context(list, Node):
         while i < hi:
             mid = (i + hi) // 2
             n = self[mid]
-            if n.end <= pos:
+            if n.pos >= pos:
+                hi = mid
+            elif n.end <= pos:
                 i = mid + 1
             else:
                 hi = mid
@@ -738,6 +748,15 @@ class Context(list, Node):
             i -= 1
             return self[i].find_token_before(pos) if self[i].is_context else self[i]
 
+    def tokens_range(self, start=0, end=None):
+        """Yield all tokens (that completely fill this text range if specified).
+
+        The first and last tokens may overlap with the start and end positions.
+
+        """
+        for context, slice_ in self.context_slices(start, end):
+            yield from util.tokens(context[slice_])
+
     def context_slices(self, start=0, end=None):
         """Yield (context, slice) tuples to yield tokens from.
 
@@ -791,9 +810,6 @@ class Context(list, Node):
         point to descendant tokens of this context. The yielded slices
         include these tokens.
 
-        The yielded slices will never be empty, in that case this method
-        will skip them.
-
         If you specify a ``target_factory``, it should be a TargetFactory
         object, and it will be updated along with the yielded slices.
 
@@ -808,12 +824,9 @@ class Context(list, Node):
                     n = n[i]
                 yield ancestors[-1][0], slice(i, None) # include start token
                 for p, i in ancestors[-2::-1]:
-                    if target_factory:
-                        target_factory.pop()
-                    if i < len(p) - 1:
-                        yield p, slice(i + 1, None)
-                if target_factory:
-                    target_factory.pop()
+                    target_factory and target_factory.pop()
+                    yield p, slice(i + 1, None)
+                target_factory and target_factory.pop()
                 start += 1
         else:
             start = 0
@@ -822,19 +835,15 @@ class Context(list, Node):
             if len(end_trail) == 1:
                 yield self, slice(start, end + 1)    # include end token
             else:
-                if start < end:
-                    yield self, slice(start, end)
+                yield self, slice(start, end)
                 n = self[end]
                 for end in end_trail[1:-1]:
-                    if target_factory:
-                        target_factory.push(n.lexicon)
-                    if end:
-                        yield n, slice(end)
+                    target_factory and target_factory.push(n.lexicon)
+                    yield n, slice(end)
                     n = n[end]
-                if target_factory:
-                    target_factory.push(n.lexicon)
+                target_factory and target_factory.push(n.lexicon)
                 yield n, slice(end_trail[-1] + 1)   # include end token
-        elif start < len(self):
+        else:
             yield self, slice(start, None)
 
     def source(self):
@@ -850,52 +859,3 @@ class Context(list, Node):
                 token = token.group[0]
             return token
 
-    def events(self, start_trail=None, end_trail=None):
-        """Yield Events for all or specified child nodes.
-
-        Using ``start_trail`` and ``end_trail`` the range of tokens can be
-        specified; a trail is a list of indices to follow to get to a specific
-        token, such as returned by the various ``find_\*_with_trail`` methods.
-
-        """
-        target = TargetFactory()
-        def events(context):
-            nodes = iter(context)
-            for n in nodes:
-                if n.is_token:
-                    tokens = (n.pos, n.text, n.action),
-                    if n.group:
-                        rest = len(n.group) - n.group.index(n) - 1
-                        tokens += tuple((t.pos, t.text, t.action)
-                            for t in itertools.islice(nodes, rest))
-                    yield Event(target.get(), tokens)
-                else:
-                    target.push(n.lexicon)
-                    yield from events(n)
-                    target.pop()
-        for context, slice_ in self.slices(start_trail, end_trail, target):
-            yield from events(context[slice_])
-
-
-def tokens(nodes):
-    """Helper to yield tokens from the iterable of nodes."""
-    for n in nodes:
-        if n.is_token:
-            yield n
-        else:
-            yield from tokens(n)
-
-
-def tokens_bw(nodes):
-    """Helper to yield tokens from the iterable in backward direction.
-
-    Make sure nodes is already in backward direction.
-
-    """
-    for n in nodes:
-        if n.is_token:
-            yield n
-        else:
-            yield from tokens_bw(reversed(n))
-
-
